;(function ($, channel, window, smartlingApi) {
    "use strict";

    smartlingApi.SmartlingContentTree = new Class({

        initialize: function (config) {
            /**
             * The content tree container
             * @type {HTMLElement}
             * @memberOf Granite.author.ui.ContentTree
             */
            this.container = config.container;

            /**
             * The Editable store used to derive the data model from
             * @type {Granite.author.Editable[]}
             * @memberOf Granite.author.ui.ContentTree
             */
            this.editables = config.editables;

            this.rootUrl ='/'

            this.urlExtension = '.tree.smartling.json?_dc=1541618291412&_charset_=utf-8';

            /**
             * The content tree data model
             * @type {Granite.author.ui.ContentTree~DataModel}
             * @memberOf Granite.author.ui.ContentTree
             */
            this.dataModel = null; // Will be instantiated in the render() method
            /**
             * The content tree representation
             * @type {Coral.Tree}
             * @memberOf Granite.author.ui.ContentTree
             */
            this.tree = null; // Will be instantiated in the render() method
            this.init();
        },
        /**
         * Renders the content tree data model in the passed container
         * @param {Granite.author.Editable[]} [editables] - The Editable store used to derive the data model from
         * @memberOf Granite.author.ui.ContentTree
         */
        render: function (editables) {
            var self = this;

            this._unbindListeners();

            this.tree = new Coral.Tree();

            if (editables) {
                this.editables = editables;
            }

            this.dataModel = this._createDataModel(this.editables);
            this.dataModel.forEach(function (element) {
                self._renderElement(element, self.tree)
            });

            this.tree._expandCollapseAll(false);

            this.container.innerHTML = "";
            this.container.appendChild(this.tree);

            this._bindListeners();
        },

        /**
         * Initializes the content tree to be rendered when the according sidepanel tab is selected
         */
        init: function() {
            var self = this;
            $.ajax({
                url: self.rootUrl + self.urlExtension
            }).done(function (data) {
                self.render(data);
            });
        },

        _renderElement: function (element, tree) {
            var self = this;

            // If element is a container
            if (element.constructor === Array) {
                var head = element[0];
                var tail = element.slice(1, element.length);

                var subTree = this._addToTree(head, tree);

                if (tail.length > 0) {
                    tail.forEach(function (tailElement) {
                        // Recursion: each element of the tail could be also a container
                        self._renderElement(tailElement, subTree);
                    });

                }
                return subTree;
            } else {
                return this._addToTree(element, tree, false);
            }
        },

        /**
         * Adds the given element to the given tree
         *
         * @param {{}} element                  - element to be added to the content tree
         * @param {Coral.Tree} tree             - Content tree
         * @param {boolean} [isContainer]       - Is the element a container
         * @private
         */
        _addToTree: function (element, tree, isContainer) {

            var item = tree.items.add({
                value: element.value,
                content: {
                    innerHTML: "<span data-translatable=\"" + element.translatable + "\"data-node-name=\"" + element.value + "\" class=\"editor-ContentTree-item\">" + element.title + "</span>"
                }
            });

            //By default all new items should be drilldown
            if (false) {
                item.setAttribute("variant", Coral.Tree.Item.variant.LEAF);
            }

            return item;
        },

        _bindListeners: function () {
            var self = this;
            var tree = this.tree;
            // Prevent looping since the Tree change listener calls the Interaction focus listener and vice versa
            var stopPropagation = false;


            // Select the corresponding tree item
            tree.on("coral-tree:change.content-tree", function (event) {
                var selection = event.detail.selection;

                if (!!selection) {
                    var translatable = $(selection).find('[data-translatable]').data('translatable');
                    $(document).trigger('sm-advanced-translation-item-select', {nodePath: self._buildNodePath(selection), translatable: translatable == true});
                } else {
                    $(document).trigger('sm-advanced-translation-item-unselect', {translatable: false});
                }
            });

            //Expand
            $(tree).on("click", ".coral-Tree-collapseExpand", function (event) {
                event.stopPropagation();
                var eventTarget = event.target;

                if (eventTarget.classList.contains('coral-Icon--chevronRight')) {
                    var parentItem = $(eventTarget).closest('coral-tree-item').get(0);

                    if (parentItem.dataset['wasExpanded'] == 'true') {
                        return;
                    }

                    var nodePath = self._buildNodePath(parentItem);

                    $.ajax({
                        url: nodePath + self.urlExtension
                    }).done(function (data) {
                        parentItem.dataset['wasExpanded'] = true;
                        if (data.length > 0) {
                            parentItem.items.clear();
                            data.forEach(function(item) {
                                parentItem.items.add({
                                   value: item.name,
                                   content: {
                                       innerHTML: "<span data-translatable=\"" + item.translatable + "\" data-node-name=\"" + item.name + "\" class=\"editor-ContentTree-item\">" + item.text + "</span>"
                                   }
                               });
                            })
                        } else {
                            parentItem.classList.add('coral-Tree-item--leaf');
                            parentItem.setAttribute('role', 'leaf');
                        }
                    });
                }
            });

        },

        _buildNodePath: function(currentItem) {

            var parent = currentItem;
            var nodeNames = [];

            nodeNames.push($(parent).find('[data-node-name]').data('node-name'));
            while (parent.parent) {
                parent = parent.parent;
                nodeNames.push($(parent).find('[data-node-name]').data('node-name'));
            }
            return this.rootUrl + nodeNames.reverse().join('/');
        },

        _unbindListeners: function () {
            channel.off("cq-interaction-focus.content-tree");

            if (this.tree) {
                this.tree.off("coral-tree:change.content-tree");
                $(tree).off("click", ".coral-Tree-collapseExpand");
            }

        },

        /**
         * @ignore
         *
         * Creates the [DataModel]{@link Granite.author.ui.ContentTree~DataModel} to be rendered in the content tree
         * @param {Array} editables - The array of all the editables loaded (see [editables]{@link Granite.author.editables})
         * @returns {Granite.author.ui.ContentTree~DataModel}
         * @private
         */
        _createDataModel: function (editables) {
            var self = this;
            var model = [];
            // Necessary due to the flat structure (we meet the parent, consider all its children, then meet the children again)
            var added = {};

            // In editables, we know that we'll meet the parent before the children
            editables.forEach(function (editable) {
                self._addToDataModel(model, editable, added);
            });

            return model;
        },

        _addToDataModel: function (model, editable, added) {
            var self = this;
            var modelData;

            if (added[editable.name]) {
                return;
            }

            if (!this._isDisplayable(editable)) {
                return;
            }

            modelData = this._adaptToDataModel(editable);
            if (modelData) {
                model.push(modelData);
                added[editable.path] = true;
            }

        },

        /**
         * Indicates if the passed Editable should be filtered out from the data model to NOT be displayed in the content tree
         * @param {Granite.author.Editable} editable The Editable to be filtered or not from the data model
         * @returns {boolean}
         * @protected
         * @memberOf Granite.author.ui.ContentTree
         */
        _isDisplayable: function (editable) {
            // Only display:
            // - editables (ignore inspectables)
            // - containers
            // - non containers that have actions defined on them (except "new" placeholders)
            return true;
        },

        /**
         * @ignore
         * Returns an [Element]{@link Granite.author.ui.ContentTree~Element} out of an [Editable]{@link Granite.author.Editable}
         * @param {Granite.author.Editable} editable - The Editable to be adapted to the data model
         * @returns {Granite.author.ui.ContentTree~Element}
         * @private
         */
        _adaptToDataModel: function (editable) {
            return {
                value: editable.name,
                title: editable.text,
                editable: editable
            };
        }
    });

}(jQuery, jQuery(document), this, SmartlingApi));
