CQ.Ext.ns("CQ.Translator");

CQ.Ext.onReady(function () {
    /**
     * Builds a Store for the dictionary content grid
     *
     * @param dictionaryPath {String} dictionary to display
     * @param languages {Array} languages in format:
     *          <code>{code: 'de', name: 'German', country: '*'}</code>
     * @returns {CQ.Ext.data.Store} data store to be used by gird
     */
    function createGridStore(dictionaryPath, languages) {
        var fields = ['key', 'string', 'comment'];
        CQ.Ext.each(languages, function (language) {
            fields.push(language.code);
        });

        return new CQ.Ext.data.JsonStore({
            root: 'translations',
            idProperty: 'key',

            url: '/apps/smartling/i18n/translator.translations.json',
            baseParams: {
                path: dictionaryPath
            },

            proxy: new CQ.Ext.data.HttpProxy({
                api: {
                    read: { url: '/apps/smartling/i18n/translator.translations.json', method: 'GET' }
                },
                headers: {"Content-Type": "application/json; charset=utf-8"}
            }),

            fields: fields
        });
    }

    /**
     * Builds a Column Model for the dictionary content grid.<br>
     *
     * Language info format:
     * <code>{code: 'de', name: 'German', country: '*'}</code>
     *
     * @param sourceLanguage {Object} source language for translation
     * @param targetLanguages {Array} of target languages
     * @returns {CQ.Ext.grid.ColumnModel} column model to be used by gird
     */
    function createColumnModel(sourceLanguage, targetLanguages) {
        var columnConfig = {
            // specify any defaults for each column
            defaults: {
                width: 120,
                sortable: true,
                renderer: toolTipRenderer
            },
            columns: [
                {
                    id: 'string',
                    header: "Key",
                    dataIndex: 'string',
                    width: 300,
                    hidden: true
                },
                {
                    header: "Source: " + sourceLanguage.name,
                    dataIndex: sourceLanguage.code,
                    hideable: false,
                    width: 300
                },
                {
                    header: "Comment",
                    dataIndex: 'comment',
                    width: 200
                }
            ]
        };

        CQ.Ext.each(targetLanguages, function (language) {
            columnConfig.columns.push({
                header: language.name,
                dataIndex: language.code
            });
        });

        return new CQ.Ext.grid.ColumnModel(columnConfig)
    }

    var grid = new CQ.Ext.grid.GridPanel({
        region: "center",
        margins: '5 5 5 5',
        loadMask: true,
        store: new CQ.Ext.data.Store(),
        cm: new CQ.Ext.grid.ColumnModel({}),
        view: new CQ.Ext.ux.grid.BufferView({
            scrollDelay: false // render rows as they come into viewable area.
        })
    });

    var filterTerm = null;
    var filterRegex = null;

    function regexQuote(str) {
        return (str + '').replace(/([\\\.\+\*\?\[\^\]\$\(\)\{\}\=\!\<\>\|\:])/g, "\\$1");
    }

    function filter(opts) {
        if (!opts) {
            filterTerm = null;
            grid.store.clearFilter(false);
            return;
        }

        filterTerm = opts.term;
        // create a regex for matching the html-escaped literal "filterTerm" string case-insensitive, global
        filterRegex = new RegExp(regexQuote(CQ.Ext.util.Format.htmlEncode(filterTerm)), "ig");

        grid.store.filterBy(function (rec, id) {
            var found = false;

            // check all fields for occurrence of the filter term
            if (filterTerm) {
                rec.fields.each(function (field) {
                    var value = rec.get(field.name);
                    if (value && value.toLowerCase().indexOf(filterTerm.toLowerCase()) >= 0) {
                        found = true;
                        return false; // stop fields.each iteration
                    }
                    return true;
                });
            } else {
                found = true;
            }
            return found;
        });
    }

    function highlightFilterTerms(value) {
        if (filterTerm && value.toLowerCase().indexOf(filterTerm.toLowerCase()) >= 0) {
            value = CQ.Ext.util.Format.htmlEncode(value);
            return value.replace(filterRegex, "<span class='highlight'>$&</span>");
        }
        return CQ.Ext.util.Format.htmlEncode(value);
    }

    function toolTipRenderer(value, metadata) {
        if (value) {
            metadata.attr = 'title="' + CQ.Ext.util.Format.htmlEncode(value) + '"';
            return highlightFilterTerms(value);
        }
        return value;
    }

    /**
     * Loads content of the specified dictionary to a grid
     *
     * @param dictionaryData object, representing selected dictionary:
     * <code>
     *     {
     *         path: '/apps/project/i18n'
     *         sourceLanguage: {
     *             code: 'en_us',
     *             name: 'English',
     *             country: 'USA'
     *         },
     *         targetLanguages: [
     *             {code: 'de', name: 'German', country: '*'}
     *             {code: 'fr', name: 'French', country: '*'}
     *         ]
     *     }
     * </code>
     */
    function loadDictionaryContent(dictionaryData) {
        var allLanguages = [dictionaryData.sourceLanguage].concat(dictionaryData.targetLanguages);

        var dictionaryStore = createGridStore(dictionaryData.path, allLanguages);
        var dictionaryColumnModel = createColumnModel(dictionaryData.sourceLanguage, dictionaryData.targetLanguages);

        grid.reconfigure(dictionaryStore, dictionaryColumnModel);
        dictionaryStore.reload({});
    }

    var dictionariesComboBox = new CQ.Ext.form.ComboBox({
        editable: false,
        width: 300,
        triggerAction: "all",
        store: new CQ.Ext.data.JsonStore({
            autoDestroy: true,
            autoLoad: true,
            url: "/services/translation/dictionaries",
            root: "paths",
            restful: true,
            idProperty: "path",
            fields: [ "path", "sourceLanguage", "targetLanguages" ]
        }),
        valueField: "path",
        displayField: "path",
        listeners: {
            select: function (combo, record) {
                loadDictionaryContent(record.data);
            }
        }
    });


    var viewport = new CQ.Ext.Viewport({
        layout: "border",
        border: true,
        defaults: {
            border: true,
            margins: '5 5 5 5'
        },
        items: [
            {
                region: "north",
                autoHeight: true,
                layout: "fit",
                bodyStyle: "border-top: 0;",
                tbar: {
                    items: [
                        "Dictionaries:", dictionariesComboBox
                    ]
                }
            },
            {
                region: "center",
                layout: "fit",
                items: {
                    layout: "border",
                    title: "Strings &amp; Translations",
                    border: false,
                    tbar: [
                        {
                            text: "Translate",
                            iconCls: "translate-icon",
                            handler: function () {
                                var dictionaryRec = dictionariesComboBox.store.getById(dictionariesComboBox.getValue());
                                var dialog = new SMARTLING.ui.dialog.SendForTranslationDialog({paths: [dictionaryRec.id]});
                                dialog.show();
                            }
                        }
                    ],
                    items: [
                        grid
                    ]
                }
            }
        ]
    });

    CQ.Translator.load = function () {
        dictionariesComboBox.store.addListener('load', function() {
            var selectedRecord = dictionariesComboBox.store.getAt(0);

            dictionariesComboBox.setValue(selectedRecord.get('path'));
            loadDictionaryContent(selectedRecord.data);
        }, this, { single : true });
    };
});